<?php
/* $Id: pivots.inc,v 1.17 2007/10/20 20:45:41 danithaca Exp $ */

/**
 * @file
 * This file includes a set of reusable functions and hook definition.
 *
 * @auther Daniel Zhou (danithaca@drupal)
 * @auther School of Information, University of Michigan.
 */


///////////////////////////////////////
//     Pivot algorithm hook definition
///////////////////////////////////////

// To write a new pivot algoritm, you need to implement the functions (hooks) defined below.
// The data structure is also defined here, which is passed by reference in most hooks.
// Please refer to pivots_magicword.* for an example.

/**
 * This is the definition of the algorithm data structure
 */
// $algorithm = array();

/**
 * The algorithm name.
 */
// $algorithm['alg_name']:

/**
 * Reference the pivot on which an instance of the algorithm is created.
 * It will only be set by pivots_algorithm_create($pivot_id).
 */
// $algorithm['pivot_id']:

/**
 * @return array
 *    the array of arguments names to be serialized/unserialized.
 */
function pivots_hook_args_list(&$algorithm) {}

/**
 * This is the default arguments serialize function, which will be used to save arguments string to database.
 * It will serialze the arguments indicated by pivots_hook_args_list().
 * Generally you don't need to implement this hook. You only need to implement pivots_hook_args_list()
 * @return string
 *    the serialized arguments.
 */
function pivots_hook_args_serialize(&$algorithm) {
  $args_list = pivots_algorithm_invoke($algorithm, 'args_list');
  if (is_null($args_list)) return;
  $arguments = array();
  foreach ($args_list as $arg) {
    $arguments[$arg] = $algorithm[$arg];
  }
  return serialize($arguments);
}

/**
 * Set the arguments of the algorithm instance based on $arg_string.
 * This is the default arguments unserialize function.
 * It will unserialze the arguments indicated by pivots_hook_args_list().
 * Generally you don't need to implement this hook. You only need to implement pivots_hook_args_list()
 *
 * @param string $arg_string
 *    the serialized string of the arguments
 */
function pivots_hook_args_unserialize(&$algorithm, $arg_string) {
  $args_list = pivots_algorithm_invoke($algorithm, 'args_list');
  if (is_null($args_list)) return;
  $arguments = unserialize($arg_string);
  foreach ($args_list as $arg) {
    $algorithm[$arg] = $arguments[$arg];
  }
}

/**
 * @return array
 *    the arugments input form for the algorithm to be displayed in the admin interface.
 *    the form will be handled by args_form_handler().
 */
function pivots_hook_args_form(&$algorithm) {}

/**
 * Handle the arguments input form generated by args_form()
 * @param array $arg_array
 *    the array of the argument input form elements
 */
function pivots_hook_args_form_handler(&$algorithm, $arg_array) {
  if (!empty($arg_array)) {
    foreach ($arg_array as $arg_key => $arg_value) {
      $algorithm[$arg_key] = $arg_value;
    }
  }
}

/**
 * Return the status of the algorithm.
 * For example, whether a pivot calculation is in progress.
 */
function pivots_hook_status(&$algorithm) {}

/**
 * Find related items of the node, return them in the ranking order.
 * This is where you want to implement the pivots matching logic.
 *
 * @param string $node_id the node id
 * @return string
 *    the actual content to be displayed in the block content, including formats.
 */
function pivots_hook_block_content(&$algorithm, $node_id) {}

/**
 * Return the block title for the pivots.
 * Default is the pivot name
 */
function pivots_hook_block_title(&$algorithm, $node_id) {
  $pivot_id = $algorithm['pivot_id'];
  $pivot = pivots_load($pivot_id);
  return check_plain($pivot->name);
}


/**
 * Find related items of the node, return them in a page.
 * Compared to pivots_hook_block_content, this is usually where you return a whole list of related items.
 * Usually it will be called when users hit the "more" link.
 *
 * @param string $node_id the node id
 * @return string
 *    the actual content to be displayed in the node, including formats and title/content.
 */
function pivots_hook_page(&$algorithm, $node_id) {}



//////////////////////////////////////////////////
//     The functions related to algorithm definition.
//////////////////////////////////////////////////

/**
 * For each algorithm, you need to register the algorithm with the module.
 * Generally, it will be called in the *.install file of the algorithm module.
 * @param string $alg_name  the identifier of the algorithm; should be unique; only letters/numbers/underscore
 * @param string $title  the readable name of the algorithm
 * @param string $description
 * @return NULL
 */
function pivots_algorithm_register($alg_name, $title, $description=NULL) {
  db_query("INSERT INTO {pivots_algorithm}(alg_name, title)
    values('%s', '%s')", $alg_name, $title);
  if (isset($description)) // this way, the description field will be left with NULL rather than an empty string.
    db_query("UPDATE {pivots_algorithm} SET description='%s' WHERE alg_name='%s'", $description, $alg_name);
}

/**
 * Get all (by default, enabled) {pivot_algorithms} records of registered algorithms.
 * @param boolean $enabled
 *    if true, only return enabled algorithm registrations; otherwise return all registrations
 * @return array
 *    with alg_name as the index
 */
function pivots_algorithm_registrations($enabled=TRUE) {
  $registrations = array();
  if ($enabled)
    $queryResult = db_query("SELECT * from {pivots_algorithm} WHERE enabled=1");
  else
    $queryResult = db_query("SELECT * from {pivots_algorithm}");
  while ($r = db_fetch_object($queryResult)) {
    $registrations[$r->alg_name] = $r;
  }
  return $registrations;
}

/**
 * Invoke the hook implementation of each algorithm.
 * If the function of a module is undefined, it will call the pivots_hook_() functions instead.
 * 
 * @param pass-by-reference &$algorithm
 * @param string $hook
 * @param mixed $a1 $a2 $a3 $a4
 * @return the result of the function implementation.
 */
function pivots_algorithm_invoke(&$algorithm, $hook, $a1=NULL, $a2=NULL, $a3=NULL, $a4=NULL) {
  $function = 'pivots_'. $algorithm['alg_name']. '_'. $hook;

  if (function_exists($function)) { // the hook is implemented
    // invoke the hook, copied from node.module -> node_invoke().
    return ($function($algorithm, $a1, $a2, $a3, $a4));
  } else {
    // invoke the default implementation.
    $function = 'pivots_hook_'. $hook;
    return ($function($algorithm, $a1, $a2, $a3, $a4));
  }
}

/**
 * Create an instance of the algorithm from a pivot_id.
 * Only this function sets the $pivot_id, and generates a valid instance of the algorithm.
 * @param integer $pivot_id.
 * @return Algorithm object
 */
function pivots_algorithm_create($pivot_id) {
  $pivot = pivots_load($pivot_id);
  $algorithm = array();
  $algorithm['pivot_id'] = $pivot_id;
  $algorithm['alg_name'] = $pivot->alg_name;
  if (isset($pivot->arguments)) {
    pivots_algorithm_invoke($algorithm, 'args_unserialize', $pivot->arguments);
  }  
  return $algorithm;
}

/**
 * Remove an algorithm when it's uninstalled.
 */
function pivots_algorithm_remove($alg_name) {
  _remove_blocks($alg_name);
  // the {pivots},{pivots_algorithm} tables might already been dropped during uninstallation.
  if (db_table_exists('{pivots}'))
    db_query("DELETE FROM {pivots} WHERE alg_name='%s'", $alg_name);
  if (db_table_exists('{pivots_algorithm}'))
    db_query("DELETE FROM {pivots_algorithm} WHERE alg_name='%s'", $alg_name);
}

/**
 * Enable an algorithm when it's enabled in the "Module" admin interface.
 */
function pivots_algorithm_enable($alg_name) {
  db_query("UPDATE {pivots_algorithm} SET enabled=1 WHERE alg_name='%s'", $alg_name);
  db_query("UPDATE {pivots} SET enabled=1 WHERE alg_name='%s'", $alg_name);
}

/**
 * Disable an algorithm when it's disabled in the "Module" admin interface.
 */
function pivots_algorithm_disable($alg_name) {
  db_query("UPDATE {pivots} SET enabled=0 WHERE alg_name='%s'", $alg_name);
  db_query("UPDATE {pivots_algorithm} SET enabled=0 WHERE alg_name='%s'", $alg_name);
  _remove_blocks($alg_name);
}


/**
 * When an algorithm is disabled or removed, we need to remove related blocks.
 */
function _remove_blocks($alg_name) {
  if (db_table_exists('{pivots}')) {
    db_query("DELETE FROM {blocks_roles} WHERE module='pivots' AND delta IN (SELECT pid from {pivots} WHERE alg_name='%s')", $alg_name);
    db_query("DELETE FROM {blocks} WHERE module='pivots' AND delta IN (SELECT pid from {pivots} WHERE alg_name='%s')", $alg_name);
  }
  else { // {pivots} does not exist, then delete all blocks related to pivots.
    db_query("DELETE FROM {blocks_roles} WHERE module='pivots'");
    db_query("DELETE FROM {blocks} WHERE module='pivots'");
  }
}


////////////////////////////////////////
//     REUSABLE FUNCTIONS
////////////////////////////////////////

/**
 * Load a pivot.
 */
function pivots_load($pivot_id) {
  $queryResult = db_query("SELECT * FROM {pivots} WHERE pid=%d;", $pivot_id);
  $pivot = db_fetch_object($queryResult); // $pivot_id is unique
  return $pivot;
}

/**
 * Delete a pivot and related stuff.
 * ATTENTION: will not clean up the Async lookup table if the pivot is using Async Algorithm.
 *     Please manually clean the lookup data or run the "clean table" function
 */
function pivots_delete($pivot_id) {
  db_query("DELETE FROM {blocks} WHERE module='pivots' AND delta=%d", $pivot_id);
  db_query("DELETE FROM {pivots} WHERE pid=%d", $pivot_id);
}

/**
 * Update the arguments field for a pivot record.
 */
function pivots_updatedb_arguments($algorithm) {
  $pivot_id = $algorithm['pivot_id'];
  if (isset($pivot_id)) {
    $arg_string = pivots_algorithm_invoke($algorithm, 'args_serialize');
    db_query("UPDATE {pivots} SET arguments='%s' WHERE pid=%d", $arg_string, $pivot_id);
  }
}

/**
 * Enclose an array to be like ('a', 'b').
 */
function pivots_enclose($array) {
  foreach ($array as $key => $element) {
    $array[$key] = "'". $element. "'";
  }
  return "(". implode(",", $array). ")";
}

/**
 * Shared theme function for showing the items in a block.
 * @param array $items
 *     the items to be listed, is not empty
 * @param string $link
 *     if $link is not set, then do not show "More"; otherwise, show 'More'.
 */
function theme_pivots_item_list_block($items, $link=NULL) {
  $block_content = theme('item_list', $items);

  if (isset($link)) {
    $block_content .= "<div class=\"more-link\">". l(t("more"), $link,
      array("title" => t("More entries.")) ). "</div>";
  }

  return $block_content;
}